#include "CeoBroj.h"
#include <iostream>
#include <cstdlib>
#include <vector>
#include <stdexcept>

using namespace std;

CeoBroj::CeoBroj()
{};

CeoBroj::CeoBroj( int n, unsigned osnova=10 )
		: _Osnova( osnova ),
		  _Znak( n>=0 ? 1 : -1 )
		{
			if(osnova>36)
			   throw invalid_argument("Suvise velika osnova");
			InicijalizacijaCifara( abs(n) );
		}

unsigned CeoBroj::Osnova() const
{
	return _Osnova;
}

int CeoBroj::Znak() const
{
	return _Znak;
}

CeoBroj CeoBroj::operator+ ( const CeoBroj& c ) const
{
	if(Osnova()!=c.Osnova())
		throw invalid_argument("Nisu iste osnove!");

	CeoBroj r;
	//ako su brojevi istog znaka
	if( Znak() == c.Znak() )
	{
		//broju koji ima vecu apsolutnu vrednost dodajemo broj sa manjom aps. vrednoscu
		if( AbsVeci( *this, c ))
			AbsSabiranje( *this, c, r );
		else
			AbsSabiranje( c, *this, r );
		r._Znak = Znak();
	}
	//ako su razlicitog znaka, od broja sa vecom aps. vred. oduzimamo  broj sa manjom aps. vred.
	else if( AbsVeci( *this, c ))
	{
		AbsOduzimanje( *this, c, r );
		r._Znak = Znak();
	}
	else
	{
		AbsOduzimanje( c, *this, r );
		r._Znak = c.Znak();
	}

	return r;
}

CeoBroj CeoBroj::operator- ( const CeoBroj& c ) const
{
	if(Osnova()!=c.Osnova())
		throw invalid_argument("Nisu iste osnove!");

	CeoBroj r;
	//ako su brojevi razlicitog znaka
	if( Znak() != c.Znak() )
	{
		//broju sa vecom aps. vred. dodajemo broj sa manjom aps. vred.
		if( AbsVeci( *this, c ))
			AbsSabiranje( *this, c, r );
		else
			AbsSabiranje( c, *this, r );
		r._Znak = Znak();
	}
	//ako su brojevi istog znaka, od broja sa vecom aps. vred. oduzimamo broj sa  manjom aps. vred.
	else if( AbsVeci( *this, c ))
	{
		AbsOduzimanje( *this, c, r );
		r._Znak = Znak();
	}
	else
	{
		AbsOduzimanje( c, *this, r );
		r._Znak = -Znak();
	}
	return r;

}

CeoBroj CeoBroj::operator* ( const CeoBroj& c ) const
{
	if(Osnova()!=c.Osnova())
		throw invalid_argument("Nisu iste osnove!");

	CeoBroj r;
	r._Osnova = _Osnova;
	r._Znak = 1;

	for( int i=_Cifre.size()-1; i>=0; i--)
	{
		// mnozimo medjurezultat osnovom
		r._Cifre.insert( r._Cifre.begin(), 0 );

		// izracunavamo jedan medjuproizvod
		CeoBroj korak;
		korak._Osnova = _Osnova;
		korak._Znak = 1;
		AbsMnozenjeCifrom( c, _Cifre[i], korak );

		// dodajemo na medjurezultat
		r = r + korak;
	}

	r._Znak = Znak() * c.Znak();
	return r;
}

bool CeoBroj::operator ==(const CeoBroj& c)  const
{
	if( !((*this)._Osnova==c._Osnova))
		return false;


	if(!((*this)._Znak==c._Znak))
		return false;

	if((*this)._Cifre.size() != c._Cifre.size())
		return false;

	for(unsigned i=0; i<(*this)._Cifre.size(); i++)
		if((*this)._Cifre[i] != c._Cifre[i])
			return false;

    return true;
}

bool CeoBroj::operator !=(const CeoBroj& c) const
{
	return ! operator==(c);
}


char CeoBroj::ZapisCifre( int c ) const
{
	string cifre = "0123456789abcdefghijklmnopqrstuwvxyz";
	return cifre[c];
}

void CeoBroj::AbsSabiranje( const CeoBroj& veci, const CeoBroj& manji, CeoBroj& r ) const
{
	unsigned osn = r._Osnova = veci._Osnova;
	unsigned prenos = 0;
	unsigned brCifaraManjeg = manji._Cifre.size();
	unsigned brCifaraVeceg = veci._Cifre.size();

	for( unsigned i=0; i<brCifaraVeceg; i++ )
	{
		unsigned c =
			veci._Cifre[i]
			+ ( brCifaraManjeg>i ? manji._Cifre[i] : 0 )
			+ prenos;
		if( c>=osn )
		{
			prenos = 1;
			c -= osn;
		}
		else
			prenos = 0;
		r._Cifre.push_back( c );
	}

	if( prenos > 0 )
		r._Cifre.push_back( prenos );
}

void CeoBroj::AbsOduzimanje( const CeoBroj& veci, const CeoBroj& manji, CeoBroj& r ) const
{
	unsigned osn = r._Osnova = veci._Osnova;
	int pozajmica = 0;
	unsigned brCifaraManjeg = manji._Cifre.size();
	unsigned brCifaraVeceg = veci._Cifre.size();

	for( unsigned i=0; i<brCifaraVeceg; i++ )
	{
		int c =
			(int)veci._Cifre[i]
			- (int)( brCifaraManjeg>i ? manji._Cifre[i] : 0 )
			- (int)pozajmica;
		if( c<0 )
		{
			pozajmica = 1;
			c += osn;
		}
		else
			pozajmica = 0;

		r._Cifre.push_back( c );
	}

	// brisemo vodece nule
	while( r._Cifre.size()>1 && r._Cifre.back() == 0 )
		r._Cifre.pop_back();
}

// da li je prvi >= drugi
bool CeoBroj::AbsVeci( const CeoBroj& x, const CeoBroj& y ) const
{
	if( x._Cifre.size() > y._Cifre.size() )
		return true;

	if( x._Cifre.size() < y._Cifre.size() )
		return false;

	for( int i=x._Cifre.size(); i>=0; i-- )
	{
		if( x._Cifre[i] > y._Cifre[i] )
			return true;

		if( x._Cifre[i] < y._Cifre[i] )
			return false;
	}

	// slucaj x==y
	return true;
}

void CeoBroj::InicijalizacijaCifara( unsigned n )
{
	_Cifre.clear();

	while( n>0 )
	{
		_Cifre.push_back( n % _Osnova );
		n /= _Osnova;
	}

	if( _Cifre.size()==0 )
		_Cifre.push_back( 0 );
}

ostream& operator<< (ostream& ostr, const CeoBroj& c )
{
	if( c._Znak<0 )
		ostr << '-';

	for( int i=c._Cifre.size()-1; i>=0; i-- )
		ostr << c.ZapisCifre( c._Cifre[i] );

	return ostr;
}


